<?php
/**
 * ZenCMS Software
 * Copyright 2012-2014 ZenThang
 * All Rights Reserved.
 *
 * This file is part of ZenCMS.
 * ZenCMS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License.
 *
 * ZenCMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with ZenCMS.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package ZenCMS
 * @copyright 2012-2014 ZenThang
 * @author ZenThang
 * @email thangangle@yahoo.com
 * @link http://zencms.vn/ ZenCMS
 * @license http://www.gnu.org/licenses/ or read more license.txt
 */
if (!defined('__ZEN_KEY_ACCESS')) exit('No direct script access allowed');

if (!function_exists("mime_content_type")) {

    function mime_content_type($fn) {

        static $mime_magic_data;

        #-- fallback
        $type = false;

        #-- read in first 3K of given file
        if (is_dir($fn)) {
            return("httpd/unix-directory");
        }
        elseif (is_resource($fn) || ($fn = @fopen($fn, "rb"))) {
            $bin = fread($fn, $maxlen=3072);
            fclose($fn);
        }
        elseif (!file_exists($fn)) {
            return false;
        }
        else {
            return("application/octet-stream");   // give up
        }

        #-- use the appropriate external tool
        /*
        if (function_exists("exec") && !ini_get("safe_mode")) {
           $type = exec("file -iL " . escapeshellcmd($fn) . " 2>/dev/null");
           if ($type = trim(strtok(substr(strrchr($type, ":"), 1), ";"))) {
               return $type;
           }
        }
        */

        #-- use PECL::fileinfo when available
        if (function_exists("finfo_buffer")) {
            $fi = new finfo(FILEINFO_MIME_TYPE);
            $type = $fi->buffer($bin);
            return($type);
        }

        $mime_magic_data = array();

        #-- read in magic data, when called for the very first time
        if (!isset($mime_content_type)) {

            if ((file_exists($fn = ini_get("mime_magic.magicfile")))
                or (file_exists($fn = "/usr/share/misc/magic.mime"))
                or (file_exists($fn = "/etc/mime-magic"))   )
            {
                $mime_magic_data = array();

                #-- read in file
                $f = fopen($fn, "r");
                $fd = fread($f, 1<<20);
                fclose($f);
                $fd = str_replace("       ", "\t", $fd);

                #-- look at each entry
                foreach (explode("\n", $fd) as $line) {

                    #-- skip empty lines
                    if (!strlen($line) or ($line[0] == "#") or ($line[0] == "\n")) {
                        continue;
                    }

                    #-- break into four fields at tabs
                    $l = preg_split("/\t+/", $line);
                    @list($pos, $typestr, $magic, $ct) = $l;
#print_r($l);

                    #-- ignore >continuing lines
                    if ($pos[0] == ">") {
                        continue;
                    }
                    #-- real mime type string?
                    $ct = strtok($ct, " ");
                    if (!strpos($ct, "/")) {
                        continue;
                    }

                    #-- mask given?
                    $mask = 0;
                    if (strpos($typestr, "&")) {
                        $typestr = strtok($typestr, "&");
                        $mask = strtok(" ");
                        if ($mask[0] == "0") {
                            $mask = ($mask[1] == "x") ? hexdec(substr($mask, 2)) : octdec($mask);
                        }
                        else {
                            $mask = (int)$mask;
                        }
                    }

                    #-- strip prefixes
                    if ($magic[0] == "=") {
                        $magic = substr($magic, 1);
                    }

                    #-- convert type
                    if ($typestr == "string") {
                        $magic = stripcslashes($magic);
                        $len = strlen($magic);
                        if ($mask) {
                            continue;
                        }
                    }
                    #-- numeric values
                    else {
                        $mask or $mask = 0xFFFFFFFF;

                        if ((ord($magic[0]) < 48) or (ord($magic[0]) > 57)) {
#echo "\nmagicnumspec=$line\n";
#var_dump($l);
                            continue;  #-- skip specials like  >, x, <, ^, &
                        }

                        #-- convert string representation into int
                        if ((strlen($magic) >= 4) && ($magic[1] == "x")) {
                            $magic = hexdec(substr($magic, 2));
                        }
                        elseif ($magic[0]) {
                            $magic = octdec($magic);
                        }
                        else {
                            $magic = (int) $magic;
                            if (!$magic) { continue; }   // zero is not a good magic value anyhow
                        }

                        #-- different types
                        switch ($typestr) {

                            case "byte":
                                $len = 1;
                                break;

                            case "beshort":
                                $magic = ($magic >> 8) | (($magic & 0xFF) << 8);
                            case "leshort":
                            case "short":
                                $len = 2;
                                break;

                            case "belong":
                                $magic = (($magic >> 24) & 0xFF)
                                    | (($magic >> 8) & 0xFF00)
                                    | (($magic & 0xFF00) << 8)
                                    | (($magic & 0xFF) << 24);
                            case "lelong":
                            case "long":
                                $len = 4;
                                break;

                            default:
                                // date, ldate, ledate, leldate, beldate, lebelbe...
                                continue;
                        }
                    }

                    #-- add to list
                    $mime_magic_data[] = array($pos, $len, $mask, $magic, trim($ct));
                }
            }
#print_r($mime_magic_data);
        }


        #-- compare against each entry from the mime magic database
        foreach ($mime_magic_data as $def) {

            #-- entries are organized as follows
            list($pos, $len, $mask, $magic, $ct) = $def;

            #-- ignored entries (we only read first 3K of file for opt. speed)
            if ($pos >= $maxlen) {
                continue;
            }

            $slice = substr($bin, $pos, $len);
            #-- integer comparison value
            if ($mask) {
                $value = hexdec(bin2hex($slice));
                if (($value & $mask) == $magic) {
                    $type = $ct;
                    break;
                }
            }
            #-- string comparison
            else {
                if ($slice == $magic) {
                    $type = $ct;
                    break;
                }
            }
        }// foreach

        #-- built-in defaults
        if (!$type) {

            #-- some form of xml
            if (strpos($bin, "<"."?xml ") !== false) {
                return("text/xml");
            }
            #-- html
            elseif ((strpos($bin, "<html>") !== false) || (strpos($bin, "<HTML>") !== false)
                || strpos($bin, "<title>") || strpos($bin, "<TITLE>")
                || (strpos($bin, "<!--") !== false) || (strpos($bin, "<!DOCTYPE HTML ") !== false)) {
                $type = "text/html";
            }
            #-- mail msg
            elseif ((strpos($bin, "\nReceived: ") !== false) || strpos($bin, "\nSubject: ")
                || strpos($bin, "\nCc: ") || strpos($bin, "\nDate: ")) {
                $type = "message/rfc822";
            }
            #-- php scripts
            elseif (strpos($bin, "<"."?php") !== false) {
                return("application/x-httpd-php");
            }
            #-- plain text, C source or so
            elseif (strpos($bin, "function ") || strpos($bin, " and ")
                || strpos($bin, " the ") || strpos($bin, "The ")
                || (strpos($bin, "/*") !== false) || strpos($bin, "#include ")) {
                return("text/plain");
            }

            #-- final fallback
            else {
                $type = "application/octet-stream";
            }
        }



        #-- done
        return $type;
    }
}


if (!function_exists("exif_imagetype") || !function_exists("image_type_to_mime_type")) {


    if (!defined('IMAGETYPE_GIF')) define("IMAGETYPE_GIF", 1);
    if (!defined('IMAGETYPE_JPEG')) define("IMAGETYPE_JPEG", 2);
    if (!defined('IMAGETYPE_PNG')) define("IMAGETYPE_PNG", 3);
    if (!defined('IMAGETYPE_SWF')) define("IMAGETYPE_SWF", 4);
    if (!defined('IMAGETYPE_PSD')) define("IMAGETYPE_PSD", 5);  // post-4.3 from here ...
    if (!defined('IMAGETYPE_BMP')) define("IMAGETYPE_BMP", 6);
    if (!defined('IMAGETYPE_TIFF_II')) define("IMAGETYPE_TIFF_II", 7);
    if (!defined('IMAGETYPE_TIFF_MM')) define("IMAGETYPE_TIFF_MM", 8);
    if (!defined('IMAGETYPE_JPC')) define("IMAGETYPE_JPC", 9);
    if (!defined('IMAGETYPE_JP2')) define("IMAGETYPE_JP2", 10);
    if (!defined('IMAGETYPE_JPX')) define("IMAGETYPE_JPX", 11);
    if (!defined('IMAGETYPE_JB2')) define("IMAGETYPE_JB2", 12);
    if (!defined('IMAGETYPE_SWC')) define("IMAGETYPE_SWC", 13);
    if (!defined('IMAGETYPE_IFF')) define("IMAGETYPE_IFF", 14);
    if (!defined('IMAGETYPE_WBMP')) define("IMAGETYPE_WBMP", 15);
    if (!defined('IMAGETYPE_XBM')) define("IMAGETYPE_XBM", 16);
    if (!defined('IMAGETYPE_MNG')) define("IMAGETYPE_MNG", 77);
    if (!defined('IMAGETYPE_XPM')) define("IMAGETYPE_XPM", 88);
    if (!defined('IMAGETYPE_ZIF')) define("IMAGETYPE_ZIF", 90);
    if (!defined('IMAGETYPE_PBM')) define("IMAGETYPE_PBM", 80);
    if (!defined('IMAGETYPE_PGM')) define("IMAGETYPE_PGM", 81);
    if (!defined('IMAGETYPE_PPM')) define("IMAGETYPE_PPM", 82);

}

#-- gives Media Type for the index numbers getimagesize() returned
if (!function_exists("image_type_to_mime_type")) {

    function image_type_to_mime_type($id) {
        static $mime = array(
            IMAGETYPE_GIF => "gif",
            IMAGETYPE_JPEG => "jpeg",
            IMAGETYPE_PNG => "png",
            IMAGETYPE_SWF => "application/x-shockwave-flash",
            IMAGETYPE_BMP => "bmp",
            IMAGETYPE_JP2 => "jp2",
            IMAGETYPE_WBMP => "vnd.wap.wbmp",
            IMAGETYPE_XBM => "xbm",
            IMAGETYPE_PSD => "x-photoshop",
            IMAGETYPE_TIFF_II => "tiff",
            IMAGETYPE_TIFF_MM => "tiff",
            IMAGETYPE_JPC => "application/octet-stream",
            IMAGETYPE_JP2 => "jp2",
//         IMAGETYPE_JPX => "",
//         IMAGETYPE_JB2 => "",
            IMAGETYPE_SWC => "application/x-shockwave-flash",
            IMAGETYPE_IFF => "iff",
            IMAGETYPE_XPM => "x-xpm",
            IMAGETYPE_ZIF => "unknown",
            IMAGETYPE_MNG => "video/mng",
            IMAGETYPE_PBM => "x-portable-bitmap",
            IMAGETYPE_PGM => "x-portable-greymap",
            IMAGETYPE_PPM => "x-portable-pixmap",
        );
        if (isset($mime[$id])) {
            $m = $mime[$id];
            strpos($m, "/") || ($m = "image/$m");
        }
        else {
            $m = "image/unknown";
        }
        return($m);
    }
}

#-- still in CVS
if (!function_exists("image_type_to_extension")) {
    function image_type_to_extension($id, $dot=true) {
        static $ext = array(
            0=>false,
            1=>"gif", 2=>"jpeg", 3=>"png",
            "swf", "psd", "bmp",
            "tiff", "tiff",
            "jpc", "jp2", "jpx", "jb2",
            "swc", "wbmp", "xbm",
            77=>"mng", 88=>"xpm", 90=>"zif",
            80=>"pbm", 81=>"pgm", 82=>"ppm",
        );
        $m = $ext[$id];
        if ($m && $dot) {
            $m = ".$m";
        }
        return($m);
    }
}


#-- we need this then, too
if (!function_exists("exif_imagetype")) {

    function exif_imagetype($fn) {

        $magic = array(
            "\211PNG" => IMAGETYPE_PNG,
            "\377\330" => IMAGETYPE_JPEG,
            "GIF89a" => IMAGETYPE_GIF,
            "GIF94z" => IMAGETYPE_ZIF,
            "FWS" => IMAGETYPE_SWF,
            "II" => IMAGETYPE_TIFF_II,
            "MM" => IMAGETYPE_TIFF_MM,
            "/* XPM" => IMAGETYPE_XPM,
            "BM" => IMAGETYPE_BMP,  // also for OS/2
            "\212MNG" => IMAGETYPE_MNG,
            "P1" => IMAGETYPE_PBM,
            "P4" => IMAGETYPE_PBM,
            "P2" => IMAGETYPE_PGM,
            "P5" => IMAGETYPE_PGM,
            "P3" => IMAGETYPE_PPM,
            "P6" => IMAGETYPE_PPM,
        );
        if ($f = @fopen($fn, "rb")) {
            $bin = fread($f, 8);
            fclose($f);
            foreach ($magic as $scn=>$id) {
                if (!strncmp($bin, $scn, strlen($scn))) {
                    return $id;
                }
            }
        }
    }
}